﻿# Primary Adobe documentation on this process: https://helpx.adobe.com/enterprise/package/help/ccp-automation.html
# XML Processing with Powershell guide: https://blogs.msdn.microsoft.com/sonam_rastogi_blogs/2014/05/14/update-xml-file-using-powershell/

<#
.SYNOPSIS
	Use Adobe web Resources to produce an XML file for automating CC packaging.
	
.DESCRIPTION
    Script that connects to adobe's web pages that document software version
    numbers as SAP codes and uses them to modify an XML template to be fed
    to the PDapp.exe packager to generate the latest versions of Adobe CC
    products.

.PARAMETER AppVersionURL
    The full URL to the adobe page that lists the SAP codes and versions of 
    CC applications that can be installed without their base applications
    being installed first.  At the time of the script's writing the URL is:
    https://helpx.adobe.com/enterprise/package/help/apps-deployed-without-their-base-versions.html

.PARAMETER PackageName
    The customized name to be used for the Adobe CC package.  This can be anything
    such as Custom, Photoshop Device Licenses, etc.

.PARAMETER DevicePool
    Override the DevicePoolName setting in the XML template.  Most useful if
    most CC applications should use device licenses while a smaller subset
    of applications have a separte license.  The majority of applications can
    use the XML template's setting and this commandline option used to set the
    DevicePoolName for the few outliers.

.PARAMETER OutputLocation
    Optional full path to override the template's location for the packages to be written.

.PARAMETER PackageApplication
    Which Adobe Application to package.  The default is All.  Valid options are:
    'All'
    'After Effects'
    'Dreamweaver'
    'Illustrator'
    'InDesign'
    'Lightroom Classic'
    'Photoshop'
    'Premiere Pro'

.PARAMETER Template
    The full path to the XML template that is modified before being fed to the Adobe PDapp executable.
    The default is .\template.xml

.PARAMETER AutomationXML
    The full path to the XML file that will be output for use by the PDapp executable.  The default
    is .\automation.xml
	
.PARAMETER Diagnostics
    Displays various text messages to the console while the script runs that
    may be useful for troubleshooting.  Normally the script runs silently with
    only a brief notification that the XML has been written when the script
    completes.


.EXAMPLE
	.\Automate-AdobeCCPackaging.ps1 -PackageName 'Adobe All' -PackageApplication 'ALL' -OutputLocation 'C:\Adobe CC' -Template '.\Adobe-Packaging-Automation_Template.xml' -AutomationXML '.\Adobe-Packaging.xml' -Diagnostics

    Creates a new XML file based on the Adobe-Packaging-Automation_Template.xml
    file and writes it to the local directory as .\Adobe-Packaging.xml.  The
    final XM file can be fed to the Adobe PDAPP executable to automate packaging
    all application in the Aobe Create Cloud suite.  The Diagnostics switch has
    been specified so verbose output will be presented; Diagnostics will be of
    questionable use.

    The following command line will execute PDAPP with the XML from the example above:

    "C:\Program Files (x86)\Common Files\Adobe\OOBE\PDApp\core\PDapp.exe" --appletID=CCP_UI --appletVersion=1.0 --workflow=ccp --automationMode=ccp_automation --pkgConfigFile=".\Adobe-Packaging.xml"

    Fully qualified paths should be used to ensure successful packaging.

.EXAMPLE
	.\Automate-AdobeCCPackaging.ps1 -AppVersionURL 'https://helpx.adobe.com/enterprise/package/help/apps-deployed-without-their-base-versions.html' -PackageName 'Adobe DreamWeaver' -PackageApplication 'Dreamweaver' -OutputLocation 'C:\Adobe CC' -Template '.\Adobe-Packaging-Automation_Template.xml' -AutomationXML '.\Adobe-Packaging.xml'

    Specifies the Adobe website to parse for product SAP codes and versions. 
    Creates a new XML file that will automate the packaging of DreamWeaver only.
    The newly created XML file can be sent to the PDAPP executable as in the
    previous example in order to automate the packaing of Dreamweaver. This 
    example is a more likely use of the script in a production environment
    as it does not use the Diagnostics switch.


.NOTES
    FileName:    Automate-AdobeCCPackaging.ps1
    Author:      Mark Van Noy
    Oganization: University of Colorado Boulder
    Created:     06/21/2018
    Updated:     N/A
    
    Version history:
    1.0.0 - (06/21/2018) Script created
#>

[CmdletBinding()]
param
(
   [parameter(Mandatory = $false, Position=0, HelpMessage = 'Specify the Adobe URL to connect to for gathering current CC application versions')]
   [string]$AppVersionURL = 'https://helpx.adobe.com/enterprise/package/help/apps-deployed-without-their-base-versions.html',

   [parameter(Mandatory = $false, Position=1, HelpMessage = 'Provide the name of the output package')]
   [string]$PackageName = '',

   [parameter(Mandatory = $false, Position=2, HelpMessage = 'Provide the name of the device pool to Override the setting in the XML template')]
   [string]$DevicePool = '',

   [parameter(Mandatory = $false, Position=3, HelpMessage = 'Provide the full path to the location CC where packages should be written')]
   [string]$OutputLocation = '',

   [parameter(Mandatory = $false, Position=4, HelpMessage = 'Application to package')]
   [ValidateSet('All', 'After Effects', 'Dreamweaver', 'Illustrator', 'InDesign', 'Lightroom Classic', 'Photoshop', 'Premiere Pro', 'Everything')]
   [String]$PackageApplication = 'All',

   [parameter(Mandatory = $false, Position=5, HelpMessage = 'Full path to the XML template')]
   [string]$Template = '.\template.xml',

   [parameter(Mandatory = $false, Position=6, HelpMessage = 'Full path to the XML output file for automation')]
   [string]$AutomationXML = '.\automation.xml',

   [parameter(Mandatory = $false, HelpMessage = "Specify the display of diagnostic text")]
   [switch]$Diagnostics
)

$web = New-Object Net.WebClient

try
{
   $response = $web.DownloadString($AppVersionURL)
}
catch
{
   Write-Host "`n`nFailed to get a response from the server."
   Write-Host "URL: ", $AppVersionURL
   Write-Host "NOT RESPONDING`n"
   Write-Host "Exiting script."

   exit(1)
}

$rows = @()
$products = @{}
$exclusiveSapCode = ''

# We only want to extract data from the first table on the webpage.  Additional tables are for legacy products.
$tablePattern = $response.Substring(($response.IndexOf('</table>') - 41), 49)

# Use Substring to extract a single HTML table row then delete that row before the next iteration of the loop
While ($response.Contains('<tr'))
{
   $rows += $response.Substring($response.IndexOf('<tr') + 4, ($response.IndexOf('</tr>') - ($response.IndexOf('<tr') + 4)))
   $response =  $response.Remove($response.IndexOf('<tr'), (($response.IndexOf('</tr>') + 5) - ($response.IndexOf('<tr'))))
   
   if ($Diagnostics)
   {
      Write-Host "End of current table string index: ", $response.IndexOf('</table>')
   }
   
   # If the HTML pattern of the end of the table no longer matches that of the first table then exit the loop.
   if (($response.Substring(($response.IndexOf('</table>') - 41), 49)) -ne $tablePattern)
   {
      if ($Diagnostics)
      {
         Write-Host "`nFirst table's pattern:`n"
         Write-Host $tablePattern
         Write-Host "`nNew table's pattern:`n"
         Write-Host $response.Substring(($response.IndexOf('</table>') - 41), 49)
         Write-Host "`n---------------------------"
         Write-Host "Exiting table reading loop."
         Write-Host "---------------------------`n"
      }
      break
   }
}

# Iterate through each row to extract the contents of each column
Foreach ($row in $rows)
{
   $columns = @()

   # use Substring to extract the columns from HTML then delete the column before the next iteration
   while ($row.Contains('<td>'))
   {
      # Load the column text into a temp variable for further processing
      $tempField = $row.Substring($row.IndexOf('<td>') + 4, ($row.IndexOf('</td>') - ($row.IndexOf('<td>') + 4)))
      
      # Clean up extraneous HTML formatting tags
      if ($tempField.Contains('<br />'))
      {
         $tempField = $tempField.Remove($tempField.IndexOf('<br />'))
      }
      # If the <strong></strong tags exist then these are table headers and can be ignored.
      if ($tempField.Contains('<strong>'))
      {
 #        $tempField = $tempField.Remove($tempField.IndexOf('<strong>'), 8)
 #        $tempField = $tempField.Remove($tempField.IndexOf('</strong>'))
         $row = $row.Remove($row.IndexOf('<td>'), (($row.IndexOf('</td>') + 5) - ($row.IndexOf('<td>'))))
         continue
      }

      $columns += $tempField
      $row = $row.Remove($row.IndexOf('<td>'), (($row.IndexOf('</td>') + 5) - ($row.IndexOf('<td>'))))
   }

   $columnNumber = 1
   $application = New-Object -TypeName PSObject

   # Build the custom object for each individual application by reconstructing the columns into the application object
   Foreach ($column in $columns)
   {
      switch ($columnNumber)
      {
         1
         {
            Add-Member -InputObject $application -MemberType NoteProperty -Name ProductName -Value $column
         }

         2
         {
            Add-Member -InputObject $application -MemberType NoteProperty -Name SapCode -Value $column
         }

         3
         {
            Add-Member -InputObject $application -MemberType NoteProperty -Name Version -Value $column
         }

         4
         {
            Add-Member -InputObject $application -MemberType NoteProperty -Name PlatformIDs -Value $column
         }
      }

      # If the 4th column is currently being processed then reset to column one and add the object to the hash/associative array
      if ($columnNumber -lt 4)
      {
         $columnNumber++
      }
      else
      {
         $columnNumber = 1
         $products.Add($application.SapCode, $application)
      }
   }
}

if ($Diagnostics)
{
   # Write out all the product information pulled from Adobe's website
   foreach ($product in $products.Values)
   {
      Write-Host "Product Name: ", $product.ProductName
      Write-Host "Sap Code: ", $product.SapCode
      Write-Host "Version", $product.Version
      Write-Host "Platform IDs for aplicable Platforms: ", $product.PlatformIDs
      Write-Host
   }
}

# !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

# Need to process RUM pages into the XML as well
#
# Relevent web pages:
#
# https://helpx.adobe.com/enterprise/package/help/ccp-automation.html
# https://helpx.adobe.com/enterprise/package/help/remote-update-manager-channel-ids.html
# https://stackoverflow.com/questions/9053573/powershell-html-parsing-get-information-from-a-website

# !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

# Read in XML File and write out changes from the CC Website

# Read in XML Configuration template
$xml = New-Object XML
$xml.PreserveWhitespace = $true
$xml.Load($Template) 

# If Packagename is defined then replace the XML template's package name with the one specified on the command line
if ($PackageName)
{
   $xml.CCPPackage.CreatePackage.packageName = $PackageName
}

# If DevicePool is defined then override the DevicePoolName setting in the XML template file wit hthe one specified on the command line
if ($DevicePool)
{
   $xml.CCPPackage.CreatePackage.devicePoolName = $DevicePool
}

# If the path to the output package location is specified then replace the path in the XML template with the command line
if ($OutputLocation)
{
   $xml.CCPPackage.CreatePackage.outputLocation = $OutputLocation
}

# Manage what to do with the XML based on the product specified on the command line
switch ($PackageApplication)
{
   'All'
   {
      # Replace XML with all updates XML
      $xml.CCPPackage.CreatePackage.ProductCategory = 'AllLatestProducts'

      # Remove all Products nodes and HDUpdates nodes
      $products =  $xml.SelectSingleNode('//Products')
      [void]$products.ParentNode.RemoveChild($products)

      $hdupdates =  $xml.SelectSingleNode('//HDUpdates')
      [void]$hdupdates.ParentNode.RemoveChild($hdupdates)

      break
   }
   'Everything'
   {
      # Everything is an undocumented switch intended to process the XML while leaving Products and HDUpdates intact
      break
   }
   default
   {
      # Add wildcards to the specified product name in order to successfully compare with like
      $App = '*' + $PackageApplication + '*'

      # Loop through all of the read in products and match the SAP Code to the specified product
      foreach ($product in $products.Values)
      {
         if ($product.ProductName -like $App)
         {
            $exclusiveSapCode = $product.SapCode
         }
      }

      # Remove all of the XML nodes that are not associated with the SAP Code
      $allProducts =  $xml.SelectSingleNode('//Products')

      foreach ($product in $allProducts.Product)
      {
         if ($product.sapCode -ne $exclusiveSapCode)
         {
            [void]$product.ParentNode.RemoveChild($product)
         }
      }

      # Remove all of the XML nodes that are not associated with the SAP Code
      $allHdUpdates =  $xml.SelectSingleNode('//HDUpdates')

      foreach ($update in $allHdUpdates.HDUpdate)
      {
         if ($update.sapCode -ne $exclusiveSapCode)
         {
            [void]$update.ParentNode.RemoveChild($update)
         }
      }

      break
   }
}

# Remove XML Comments (https://stackoverflow.com/questions/16820175/how-can-i-comment-out-a-node-in-xml-using-powershell)
$comments = $xml.SelectNodes('//comment()')
foreach ($comment in $comments)
{
   if ($Diagnostics)
   {
      Write-Host "Removing XML Comment:"
      Write-Host "`t", $comment.OuterXML
   }

   [void]$comment.ParentNode.RemoveChild($comment)
}

# Print out the XML changes made to versions
if ($Diagnostics)
{
   Write-Host "PRODUCTS:"
   Write-Host "SAP`tSource`t`tUpdated"
   Write-Host "-----`t------`t`t-------"
}

# Loop through the XML product SAP codes and look up the current information from that stored by the web page.
foreach ($product in $xml.CCPPackage.CreatePackage.Products.Product)
{
   if ($Diagnostics)
   {
      Write-Host -NoNewline $product.sapCode, "`t", $product.Version
      if ($product.Version.Count -lt 6)
      {
         Write-Host -NoNewLine "`t`t"
      }
      else
      {
         Write-Host -NoNewLine "`t"
      }
   }

   # Set the product version based on the information read in from the website
   $product.Version = $products[$product.sapCode].Version

   if ($Diagnostics)
   {
      Write-Host $products[$product.sapCode].Version
   }
}

if ($Diagnostics)
{
   Write-Host "HD Update:"
   Write-Host "SAP`tSource`t`tUpdated"
   Write-Host "-----`t------`t`t-------"
}

# Loop through the XML HDUpdate SAP codes and look up the current information from that stored by the web page.
foreach ($hdupdate in $xml.CCPPackage.CreatePackage.HDUpdates.HDUpdate)
{
   if ($Diagnostics)
   {
      Write-Host -NoNewline $hdupdate.sapCode, "`t", $hdupdate.Version
      if ($hdupdate.Version.Count -lt 6)
      {
         Write-Host -NoNewLine "`t`t"
      }
      else
      {
         Write-Host -NoNewLine "`t"
      }
   }

   # Set the product version based on the information read in from the website
   $hdupdate.Version = $products[$hdupdate.sapCode].Version

   if ($Diagnostics)
   {
      Write-Host $products[$hdupdate.sapCode].Version
   }
}

# Remove blank lines using the writable InnerXML Method and RegEx
$xml.InnerXML = $xml.InnerXML -replace "\s+`r`n", "`r`n"

if ($Diagnostics)
{
   Write-Host "`n`nThe Modified XML File:`n"
   Write-Host $xml.OuterXML
}

# Make sure the XML file is normalized
$xml.Normalize()

# Save out the changes
$xml.Save($AutomationXML)

# Let the user know that the XML has been written
Write-Host "`nSuccessfully wrote the updated XML file to:"
Write-Host $AutomationXML